﻿using System;
using Vintagestory.API.Client;
using Vintagestory.API.Common;
using Vintagestory.API.Server;
using Vintagestory.API.Config;
using Vintagestory.API.Common.Entities;
using VSSurvivalMod;
using Vintagestory.GameContent;
using Vintagestory.GameContent.Mechanics;
using Vintagestory.API.Datastructures;
using Vintagestory.API.MathTools;
using Vintagestory.API.Util;
using System.Collections.Generic;
using System.Linq;

[assembly: ModInfo( "VesselPatch",
	Description = "Dramatically improves the drops from cracked vessels.",
	Website     = "TBR",
	Authors     = new []{ "OmegaHaxors" } )]

namespace OffhandDebuffPatch
{
	public class VesselPatch : ModSystem
	{

		ICoreAPI api = null;
		ICoreServerAPI sapi = null;
		ICoreClientAPI capi = null;

		public override void Start(ICoreAPI api)
		{
			this.api = api;
			api.RegisterBlockClass("BuffedBlockLootVessel",typeof(BuffedBlockLootVessel));
		}
		public override void StartServerSide(ICoreServerAPI sapi)
		{
			this.sapi = sapi;
		}
		public override void StartClientSide(ICoreClientAPI capi){this.capi = capi;}
	}

	public class BuffedBlockLootVessel : BlockLootVessel
	{
		new public static Dictionary<string, LootList> lootLists = new Dictionary<string, LootList>();
		static BuffedBlockLootVessel()
		{
			BuffedBlockLootVessel.lootLists["seed"] = LootList.Create(1f, new LootItem[]
			{
				LootItem.Item(1f, 5f, 7f, new string[]
				{
					"seeds-carrot",
					"seeds-onion",
					"seeds-spelt",
					"seeds-turnip",
					"seeds-rice",
					"seeds-rye",
					"seeds-soybean",
					"seeds-pumpkin",
					"seeds-cabbage"
				})
			});
			BuffedBlockLootVessel.lootLists["food"] = LootList.Create(4f, new LootItem[]
			{
				LootItem.Item(3f, 8f, 15f, new string[]
				{
					"grain-spelt",
					"grain-rice",
					"grain-flax",
					"grain-rye"
				}),
				LootItem.Item(0.1f, 1f, 1f, new string[]
				{
					"resonancearchive-1",
					"resonancearchive-2",
					"resonancearchive-3",
					"resonancearchive-4",
					"resonancearchive-5",
					"resonancearchive-6",
					"resonancearchive-7",
					"resonancearchive-8",
					"resonancearchive-9"
				})
			});
			BuffedBlockLootVessel.lootLists["forage"] = LootList.Create(2.25f, new LootItem[]
			{
				LootItem.Item(4f, 44f, 84f, new string[]
				{
					"stick",
					"drygrass"
				}),
				LootItem.Item(0.25f, 16f, 32f, new string[]
				{
					"stone-obsidian"
				}),
				LootItem.Item(0.25f, 44f, 84f, new string[]
				{
					"clay-blue",
					"clay-fire"
				}),
				LootItem.Item(0.25f, 16f, 52f, new string[]
				{
					"stone-andesite",
					"stone-chalk",
					"stone-claystone",
					"stone-granite",
					"stone-sandstone",
					"stone-shale"
				}),
				LootItem.Item(0.25f, 16f, 52f, new string[]
				{
					"cattailtops"
				}),
				LootItem.Item(4f, 20f, 28f, new string[]
				{
					"flaxfibers"
				}),
				LootItem.Item(1f, 3f, 6f, new string[]
				{
					"honeycomb",
					"bamboostakes"
				}),
				LootItem.Item(0.01f, 4f, 4f, new string[]
				{
					"beenade-closed",
					"poultice-linen-honey-sulfur"
				})
			});
			BuffedBlockLootVessel.lootLists["ore"] = LootList.Create(3.25f, new LootItem[]
			{
				LootItem.Item(0.25f, 16f, 32f, new string[]
				{
					"ore-lignite",
					"ore-bituminouscoal"
				}),
				LootItem.Item(0.5f, 8f, 24f, new string[]
				{
					"ore-quartz"
				}),
				LootItem.Item(2f, 10f, 30f, new string[]
				{
					"nugget-nativecopper"
				}),
				LootItem.Item(1f, 2f, 3f, new string[]
				{
					"nugget-sphalerite",
					"nugget-bismuthinite"
				}),
				LootItem.Item(0.25f, 5f, 15f, new string[]
				{
					"nugget-galena",
					"nugget-chromite",
					"nugget-ilmenite"
				}),
				LootItem.Item(2f, 5f, 15f, new string[]
				{
					"nugget-limonite",
					"nugget-magnetite"
				}),
				LootItem.Item(0.25f, 2f, 6f, new string[]
				{
					"nugget-cassiterite",
					"nugget-nativegold",
					"nugget-nativesilver"
				})
			});
			BuffedBlockLootVessel.lootLists["tool"] = LootList.Create(1.1f, new LootItem[]
			{
				LootItem.Item(0.5f, 1f, 1f, new string[]
				{
					"axe-obsidian",
					"shovel-obsidian",
					"knife-generic-obsidian"
				}),
				LootItem.Item(0.2f, 1f, 1f, new string[]
				{
					"axe-felling-copper",
					"shovel-copper",
					"scythe-copper",
					"knife-generic-copper",
					"blade-falx-copper"
				}),
				LootItem.Item(0.05f, 1f, 1f, new string[]
				{
					"pickaxe-copper"
				}),
				LootItem.Item(0.1f, 1f, 1f, new string[]
				{
					"axe-felling-tinbronze",
					"shovel-tinbronze",
					"scythe-tinbronze",
					"knife-generic-tinbronze",
					"blade-falx-tinbronze"
				}),
				LootItem.Item(0.15f, 4f, 12f, new string[]
				{
					"gear-rusty"
				})
			});
			BuffedBlockLootVessel.lootLists["farming"] = LootList.Create(2.25f, new LootItem[]
			{
				LootItem.Item(0.1f, 1f, 1f, new string[]
				{
					"linensack"
				}),
				LootItem.Item(0.75f, 64f, 64f, new string[]
				{
					"feather"
				}),
				LootItem.Item(0.75f, 40f, 56f, new string[]
				{
					"flaxfibers"
				}),
				LootItem.Item(0.35f, 10f, 15f, new string[]
				{
					"flaxtwine"
				}),
				LootItem.Item(0.75f, 2f, 4f, new string[]
				{
					"seeds-cabbage"
				}),
				LootItem.Item(0.75f, 2f, 3f, new string[]
				{
					"cattailroot"
				}),
				LootItem.Item(0.1f, 1f, 1f, new string[]
				{
					"scythe-copper",
					"scythe-tinbronze"
				})
			});
			BuffedBlockLootVessel.lootLists["arcticsupplies"] = LootList.Create(1.25f, new LootItem[]
			{
				LootItem.Item(1.5f, 64f, 64f, new string[]
				{
					"cattailtops"
				}),
				LootItem.Item(1.5f, 64f, 64f, new string[]
				{
					"drygrass"
				}),
				LootItem.Item(1f, 64f, 64f, new string[]
				{
					"flint"
				}),
				LootItem.Item(1f, 64f, 64f, new string[]
				{
					"stick"
				}),
				LootItem.Item(1f, 64f, 64f, new string[]
				{
					"stone-andesite",
					"stone-chalk",
					"stone-granite",
					"stone-basalt"
				}),
				LootItem.Block(1f, 64f, 64f, new string[]
				{
					"soil-medium-none"
				}),
				LootItem.Item(1f, 64f, 64f, new string[]
				{
					"clay-blue",
					"clay-fire"
				}),
				LootItem.Item(0.5f, 16f, 16f, new string[]
				{
					"poultice-linen-horsetail"
				}),
				LootItem.Item(0.5f, 40f, 56f, new string[]
				{
					"flaxfibers"
				}),
				LootItem.Item(0.3f, 3f, 9f, new string[]
				{
					"honeycomb"
				}),
				LootItem.Item(0.2f, 4f, 12f, new string[]
				{
					"gear-rusty"
				})
			});
		}
		public override float OnGettingBroken(IPlayer player, BlockSelection blockSel, ItemSlot itemslot, float remainingResistance, float dt, int counter)
		{
			return base.OnGettingBroken(player,blockSel,itemslot,remainingResistance,dt,counter);
		}
		public override void OnBlockBroken(IWorldAccessor world, BlockPos pos, IPlayer byPlayer, float dropQuantityMultiplier = 1f)
		{
			base.OnBlockBroken(world,pos,byPlayer,dropQuantityMultiplier);
		}
		public override BlockDropItemStack[] GetDropsForHandbook(ItemStack handbookStack, IPlayer forPlayer)
		{
			LootList list = BlockLootVessel.lootLists[base.LastCodePart(0)];
			List<BlockDropItemStack> drops = new List<BlockDropItemStack>();
			foreach (LootItem val in list.lootItems)
			{
				for (int i = 0; i < val.codes.Length; i++)
				{
					ItemStack lstack = val.GetItemStack(this.api.World, i, 1f);
					if (lstack == null)
					{
						AssetLocation code = val.codes[i % val.codes.Length];
						this.api.World.Logger.Warning("Unable to resolve loot vessel drop {0}, item wont drop.", new object[]
						{
							code
						});
					}
					else
					{
						BlockDropItemStack stack = new BlockDropItemStack(lstack, 1f);
						if (stack != null)
						{
							stack.Quantity.avg = val.chance / list.TotalChance / (float)val.codes.Length;
							if (drops.FirstOrDefault((BlockDropItemStack dstack) => dstack.ResolvedItemstack.Equals(this.api.World, stack.ResolvedItemstack, GlobalConstants.IgnoredStackAttributes)) == null)
							{
								drops.Add(stack);
							}
						}
					}
				}
			}
			return drops.ToArray();
		}
		public override ItemStack[] GetDrops(IWorldAccessor world, BlockPos pos, IPlayer byPlayer, float dropQuantityMultiplier = 1f)
		{
			float selfdropRate = ((byPlayer != null) ? byPlayer.Entity.Stats.GetBlended("wholeVesselLootChance") : 0f) - 1f;
			if (this.api.World.Rand.NextDouble() < (double)selfdropRate)
			{
				return new ItemStack[]
				{
					new ItemStack(this, 1)
				};
			}
			LootList list = BuffedBlockLootVessel.lootLists[base.LastCodePart(0)];
			if (list == null)
			{
				return new ItemStack[0];
			}
			return list.GenerateLoot(world, byPlayer);
		}

	}

}
